@@ -1,7 +1,6 @@
 import time
 import sys
 import re
-import os
 import json
 from pathlib import Path
 
@@ -18,51 +17,122 @@ from sepal_ui.scripts import utils as su
 su.init_ee()
 
 def display_asset(output, asset):
-    """remove the manifest from the asset name and display it to the user"""
+    """
+    remove the manifest from the asset name and display it to the user in an output
+    
+    Args: 
+        output (sw.Alert): the alert to display information to the user
+        asset (str): the asset name
+    
+    Return:
+        (str): the asset name without the manifest
+    """
     
     asset = asset.replace('projects/earthengine-legacy/assets/', '')
     
     output.add_msg(ms.aoi_sel.asset_created.format(asset), 'success')
     
-    return
+    return asset
 
 def isAsset(asset_descripsion, folder):
-    """Check if the asset already exist in the user asset folder
+    """
+    Check if the asset already exist in the user asset folder
     
     Args: 
         asset_descripsion (str) : the descripsion of the asset
         folder (str): the folder of the glad assets
         
-    Returns:
-        exist (bool): true if already in folder
+    Return:
+        (bool): true if already in folder
     """
     exist = False
     liste = ee.data.listAssets({'parent': folder})['assets']
     for asset in liste:
-        if asset['name'] == os.path.join(folder,asset_descripsion):
+        if asset['name'] == str(Path(folder,asset_descripsion)):
             exist = True
             break
             
     return exist 
 
-def get_country_asset(country_selection, output):
-    """send a request to GEE to get a featurecollection based on the country selected"""
+def get_admin0_asset(adm0, output):
+    """
+    send a request to GEE to get a featurecollection based on the country selected
     
-    if country_selection == None:
+    Args:
+        adm0 (int): the administrative level 0 in FAO GAUL 2015
+        output (sw.Alert): the alert to display information to the end user
+        
+    return:
+        (ee.FeatureColection): the FeatureCollection invocation of the country
+    """
+    
+    if adm0 == None:
         output.add_msg(ms.aoi_sel.no_country, 'warning')
         return (None, None)
     
-    country_code = utils.get_gaul_dic()[country_selection] 
-    iso_3 = utils.get_iso_3(country_selection)
+    country = ee.FeatureCollection("FAO/GAUL/2015/level0").filter(ee.Filter.eq('ADM0_CODE', adm0))
     
-    country = ee.FeatureCollection("FAO/GAUL/2015/level0").filter(ee.Filter.eq('ADM0_CODE', country_code))
-          
-    display_asset(output, iso_3)
+    #output.add_msg(ms.aoi_sel.valid_admin0.format(iso_3), 'success')
     
-    return country, iso_3
+    return country
+
+def get_admin1_asset(adm1, output):
+    """
+    send a request to GEE to get a featurecollection based on the country selected
+    
+    Args:
+        adm1 (int): the administrative level 1 in FAO GAUL 2015
+        output (sw.Alert): the alert to display information to the end user
+        
+    return:
+        (ee.FeatureColection): the FeatureCollection invocation of the country
+    """
+    
+    if adm1 == None:
+        output.add_msg(ms.aoi_sel.no_country, 'warning')
+        return (None, None)
+    
+    country = ee.FeatureCollection("FAO/GAUL/2015/level1").filter(ee.Filter.eq('ADM1_CODE', adm1))
+    
+    #
+    
+    return country
+
+def get_admin2_asset(adm2, output):
+    """
+    send a request to GEE to get a featurecollection based on the country selected
+    
+    Args:
+        adm2 (int): the administrative level 2 in FAO GAUL 2015
+        output (sw.Alert): the alert to display information to the end user
+        
+    return:
+        (ee.FeatureColection): the FeatureCollection invocation of the country
+    """
+    
+    if adm2 == None:
+        output.add_msg(ms.aoi_sel.no_country, 'warning')
+        return (None, None)
+    
+    country = ee.FeatureCollection("FAO/GAUL/2015/level2").filter(ee.Filter.eq('ADM2_CODE', adm2))
+    
+    #output.add_msg(ms.aoi_sel.valid_admin0.format(iso_3), 'success')
+    
+    return country
 
 def get_drawn_shape(drawn_feat, file_name, folder, output):
-    """send the requested drawn feature to GEE to create an asset"""
+    """
+    send the requested drawn feature to GEE to create an asset
+    
+    Args:
+        drawn_feat (ee.FeatureCollection): the feature drawn on the map
+        file_name (str): the name to use as GEE asset
+        folder (str): the path to the user GEE folder
+        output (sw.Alert): the alert to display information to the end user
+        
+    Return:
+        (str): the created asset name
+    """
     
     aoi = drawn_feat 
     asset_name = ms.aoi_sel.file_pattern.format(re.sub('[^a-zA-Z\d\-\_]', '_', file_name))
@@ -74,9 +144,9 @@ def get_drawn_shape(drawn_feat, file_name, folder, output):
         output.add_msg(ms.aoi_sel.name_used, 'error')
         return None 
         
-    asset = os.path.join(folder, asset_name)
+    asset = str(Path(folder, asset_name))
             
-    #create and launch the task
+    # create and launch the task
     task_config = {
         'collection': drawn_feat, 
         'description':asset_name,
@@ -91,7 +161,16 @@ def get_drawn_shape(drawn_feat, file_name, folder, output):
     return asset
 
 def get_gee_asset(asset_name, output):
-    """check that the asset exist return None if not"""
+    """
+    check that the asset name is not null or None
+    
+    Args:
+        asset_name (str): the asset name
+        output (sw.Alert): the alert to display information to the end user
+        
+    Return:
+        (str): return the asset if not null
+    """
     
     if asset_name == '' or asset_name == None:
         output.add_msg(ms.aoi_sel.no_asset, 'error') 
@@ -103,9 +182,20 @@ def get_gee_asset(asset_name, output):
     return asset
 
 def get_csv_asset(json_csv, file_name, folder, output):
-    """Send a request to GEE to create an asset based on the local shapefile"""
+    """
+    Send a request to GEE to create an asset based on the local shapefile
     
-    #read the json 
+    Args:
+        json_csv (str): the csv file information stored as a json dict {pathname, id, lng, lat}
+        file_name (str): the name that will be used in GEE 
+        folder (str): The user folder in GEE
+        outpput (sw.Alert): the alert to display information to the end user
+        
+    Return:
+        (str): the created asset name
+    """
+    
+    # read the json 
     load_df = json.loads(json_csv)
     
     # check that the columns are well set 
@@ -132,9 +222,9 @@ def get_csv_asset(json_csv, file_name, folder, output):
     ee_object = geemap.geojson_to_ee(json_df)
     
     # upload this object to earthengine
-    asset = os.path.join(folder, asset_name)
+    asset = str(Path(folder, asset_name))
             
-    #create and launch the task
+    # create and launch the task
     task_config = {
         'collection': ee_object, 
         'description':asset_name,
@@ -150,9 +240,23 @@ def get_csv_asset(json_csv, file_name, folder, output):
     
             
 def get_shp_aoi(file_input, file_name, folder, output):
-    """send a request to GEE to create an asset based on the local shapefile"""
+    """
+    Send a request to GEE to create an asset based on the local shapefile
     
-    if not os.path.isfile(file_input):
+    Args:
+        file_input (str | pathlib.path): the path to the .shp file
+        file_name (str): the name used for the GEE asset
+        folder (str): the user folder in GEE
+        output (sw.Alert): the alert to display information to the end user
+        
+    Return:
+        (str): the created asset name
+    """
+    
+    if type(file_input) == str:
+        file_input = Path(file_input)
+        
+    if not file_input.is_file():
         output.add_msg(ms.aoi_sel.error_occured, 'error')
         return None
     
@@ -167,16 +271,16 @@ def get_shp_aoi(file_input, file_name, folder, output):
        
     asset_name = ms.aoi_sel.file_pattern.format(re.sub('[^a-zA-Z\d\-\_]','_',file_name))
         
-    #check asset's name
+    # check asset's name
     if isAsset(asset_name, folder):
         output.add_msg(ms.aoi_sel.name_used, 'error')
         asset = None
         
     else:
         
-        asset = os.path.join(folder, asset_name)
+        asset = str(Path(folder, asset_name))
             
-        #launch the task
+        # launch the task
         task_config = {
             'collection': ee_object, 
             'description':asset_name,
@@ -191,27 +295,29 @@ def get_shp_aoi(file_input, file_name, folder, output):
     return asset
     
 def run_aoi_selection(output, list_method, io, folder=None):
-    """ create an gee asset according to the user inputs
+    """
+    Create an gee asset according to the user inputs
 
     Args:
-        file_input (str): the file to retreive from the user folder. It must be a .shp file
-        file_name (str): name of the aoi that will be used troughout the process
-        country_selection (str): a country name in english available in LSIB 2017
-        asset_name (str): the assetId of a existing asset
-        drawing_method (str): the name of the method selected to create the asset
+        io.file_input (str): the file to retreive from the user folder. It must be a .shp file
+        io.file_name (str): name of the aoi that will be used troughout the process
+        io.country_selection (str): a country name in english available in LSIB 2017
+        io.asset_name (str): the assetId of a existing asset
+        io.drawing_method (str): the name of the method selected to create the asset
         output (v.Alert): the widget used to display the process informations
         list_method ([str]): list of the method use to select an AOI
         drawn_feat (ee.FeatureCollection): the last drawn object on the map
+        folder (str, optional): the user GEE asset folder
         
-    Returns:
-        asset (str) : the AssetId of the AOI
+    Return:
+        (str) : the AssetId of the AOI
     """
-    #go to the glad folder in gee assets 
+    # go to the glad folder in gee assets 
     if not folder: 
         folder = ee.data.getAssetRoots()[0]['id']
     
-    #clean all but the selected method
-    if io.country_selection:
+    # clean all but the selected method
+    if io.adm0:
         io.assetId = None
     else:
         io.feature_collection, io.country_code = (None, None)
@@ -221,21 +327,33 @@ def run_aoi_selection(output, list_method, io, folder=None):
     
     # not selected
     if io.selection_method == None:
-        output.add_msg(ms.aoi_sel.no_selection, 'error')   
+        raise Exception(ms.aoi_sel.no_selection)   
     # use a country boundary
     elif io.selection_method == list_method[0]: 
-        io.feature_collection, io.country_code = get_country_asset(io.country_selection, output)
-    # draw a shape
+        asset = get_admin0_asset(io.adm0, output)
+        io.set_admin(asset, admin0=io.adm0)
+        output.add_msg(ms.aoi_sel.valid_admin0.format(io.get_aoi_name()), 'success')
+    # use adm1
     elif io.selection_method == list_method[1]: 
-        io.assetId = get_drawn_shape(io.drawn_feat, io.file_name, folder, output)
-    # use GEE asset
-    elif io.selection_method == list_method[3]: 
-        io.assetId = get_gee_asset(io.assetId, output)
-    # upload file
+        asset = get_admin1_asset(io.adm1, output)
+        io.set_admin(asset, admin0=io.adm0, admin1=io.adm1)
+        output.add_msg(ms.aoi_sel.valid_admin0.format(io.get_aoi_name()), 'success')
+    # use adm2
     elif io.selection_method == list_method[2]: 
-        io.assetId = get_shp_aoi(io.file_input, io.file_name, folder, output)
-    # csv point file
+        asset = get_admin2_asset(io.adm2, output)
+        io.set_admin(asset, admin0=io.adm0, admin1=io.adm1, admin2=io.adm2)
+        output.add_msg(ms.aoi_sel.valid_admin0.format(io.get_aoi_name()), 'success')
+    # draw a shape
+    elif io.selection_method == list_method[3]: 
+        io.set_asset(get_drawn_shape(io.drawn_feat, io.file_name, folder, output))
+    # use GEE asset
     elif io.selection_method == list_method[4]: 
-        io.assetId = get_csv_asset(io.json_csv, io.file_name, folder, output)
+        io.set_asset(get_gee_asset(io.assetId, output))
+    # upload file
+    elif io.selection_method == list_method[5]: 
+        io.set_asset(get_shp_aoi(io.file_input, io.file_name, folder, output))
+    # csv point file
+    elif io.selection_method == list_method[6]: 
+        io.set_asset(get_csv_asset(io.json_csv, io.file_name, folder, output))
             
     return
\ No newline at end of file
