@@ -1,9 +1,10 @@
-import os 
 from pathlib import Path
 import json
 
 import ipyvuetify as v
-from traitlets import HasTraits, Unicode, link
+from traitlets import (
+    HasTraits, Unicode, link, Int
+)
 from ipywidgets import jslink
 import pandas as pd
 import ee
@@ -17,7 +18,16 @@ from sepal_ui.sepalwidgets.btn import Btn
 su.init_ee()
 
 class DatePicker(v.Layout, SepalWidget):
+    """
+    Custom input widget to provide a reusable DatePicker. It allows to choose date as a string in the following format YYYY-MM-DD
     
+    Args:
+        label (str, optional): the label of the datepicker field
+        
+    Attributes:
+        menu (v.Menu): the menu widget to display the datepicker
+        
+    """
     def __init__(self, label="Date", **kwargs):
         
         date_picker = v.DatePicker(
@@ -66,11 +76,32 @@ class DatePicker(v.Layout, SepalWidget):
         #date_text.observe(lambda _: setattr(self.menu, 'value', False), 'v_model')
         
 class FileInput(v.Flex, SepalWidget, HasTraits):
+    """
+    Custom input field to select a file in the sepal folders. 
+    
+    Args:
+        extentions ([str]): the list of the allowed extentions. the FileInput will only display thiese extention and folders
+        folder (str | pathlib.Path): the starting folder of the file input
+        label (str): the label of the input
+        v_model (str, optional): the default value
+        
+    Attributes:
+        extentions ([str]): the extention list
+        folder (str | pathlib.Path): the current folder
+        selected_file (v.TextField): the textfield where the file pathname is stored
+        loading (v.ProgressLinear): loading top bar of the menu component
+        file_list (v.List): the list of files and folder that are available in the current folder
+        file_menu (v.Menu): the menu that hide and show the file_list
+        reload (v.Btn): reload btn to reload the file list on the current folder 
+    """
 
     file = Unicode('')
     
-    def __init__(self, extentions = [], folder=os.path.expanduser('~'), label='search file', v_model = None, **kwargs):
-
+    def __init__(self, extentions = [], folder=Path('~').expanduser(), label='search file', v_model = None, **kwargs):
+        
+        if type(folder) == str:
+            folder = Path(folder)
+            
         self.extentions = extentions
         self.folder = folder
         
@@ -136,29 +167,35 @@ class FileInput(v.Flex, SepalWidget, HasTraits):
         self.reload.on_event('click', self._on_reload)
         
     def _on_file_select(self, change):
-        new_value = change['new']
-        if new_value:
-            if os.path.isdir(new_value):
-                self.folder = new_value
-                self._change_folder()
+        """Dispatch the behaviour between file selection and folder change"""
+        
+        if not change['new']:
+            return self
+        
+        new_value = Path(change['new'])
+        
+        if new_value.is_dir():
+            self.folder = new_value
+            self._change_folder()
                 
-            elif os.path.isfile(new_value):
-                self.file = new_value
+        elif new_value.is_file():
+            self.file = str(new_value)
             
-            return
+        return self
                 
     def _change_folder(self):
-        """change the target folder"""
+        """Change the target folder"""
         #reset files
         self.file_list.children[0].children = self._get_items()
-    
-
-    def _get_items(self):
-        """return the list of items inside the folder"""
-
-        self.loading.indeterminate = not self.loading.indeterminate
         
-        folder = Path(self.folder)
+        return
+    
+    def _get_items(self):
+        """Return the list of items inside the folder"""
+
+        self.loading.indeterminate = True
+        
+        folder = self.folder
 
         list_dir = [el for el in folder.glob('*/') if not el.name.startswith('.')]
 
@@ -192,19 +229,18 @@ class FileInput(v.Flex, SepalWidget, HasTraits):
         folder_list = sorted(folder_list, key=lambda x: x.value)
         file_list = sorted(file_list, key=lambda x: x.value)
 
-        parent_path = str(folder.parent)
         parent_item = v.ListItem(
-            value=parent_path, 
+            value=str(folder.parent), 
             children=[
                 v.ListItemAction(children=[v.Icon(color=ICON_TYPES['PARENT']['color'], children=[ICON_TYPES['PARENT']['icon']])]),
-                v.ListItemContent(children=[v.ListItemTitle(children=[f'..{parent_path}'])]),
+                v.ListItemContent(children=[v.ListItemTitle(children=[f'..{folder.parent}'])]),
             ]
         )
 
         folder_list.extend(file_list)
         folder_list.insert(0,parent_item)
 
-        self.loading.indeterminate = not self.loading.indeterminate
+        self.loading.indeterminate = False
         
         return folder_list
     
@@ -216,6 +252,17 @@ class FileInput(v.Flex, SepalWidget, HasTraits):
         return
 
 class LoadTableField(v.Col, SepalWidget):
+    """
+    A custom input widget to load points data. The user will provide a csv or txt file containing labeled dataset. 
+    The relevant columns (lat, long and id) can then be identified in the updated select. Once everything is set, the widget will populate itself with a json dict.
+    {pathname, id_column, lat_column,lng_column}
+    
+    Attributes:
+        fileInput (sw.FileInput): the file input to select the .csv or .txt file
+        IdSelect (v.Select): input to select the id column
+        LngSelect (v.Select): input to select the lng column
+        LatSelect (v.Select): input to select the lat column
+    """
     
     default_v_model = {
             'pathname'  : None, 
@@ -227,10 +274,25 @@ class LoadTableField(v.Col, SepalWidget):
     def __init__(self):
         
         self.fileInput = FileInput(['.csv', '.txt'])
-                
-        self.IdSelect = self._LocalSelect('id_column', 'Id')
-        self.LngSelect = self._LocalSelect('lng_column', 'Longitude')
-        self.LatSelect = self._LocalSelect('lat_column', 'Latitude')
+        
+        self.IdSelect = v.Select(
+            _metadata = {'name': 'id_column'}, 
+            items     = [], 
+            label     = 'Id', 
+            v_model   = None
+        ) 
+        self.LngSelect = v.Select(
+            _metadata = {'name': 'lng_column'}, 
+            items     = [], 
+            label     = 'Longitude', 
+            v_model   = None
+        )
+        self.LatSelect = v.Select(
+            _metadata = {'name': 'lat_column'}, 
+            items     = [], 
+            label     = 'Latitude', 
+            v_model   = None
+        )
         
         super().__init__(
             v_model = json.dumps(self.default_v_model),
@@ -253,7 +315,7 @@ class LoadTableField(v.Col, SepalWidget):
         self.LatSelect.observe(self._on_select_change, 'v_model')
         
     def _on_file_input_change(self, change):
-        
+        """Update the select content when the fileinput v_model is changing"""
         path = change['new']
             
         df = pd.read_csv(path, sep=None, engine='python')
@@ -278,65 +340,98 @@ class LoadTableField(v.Col, SepalWidget):
                 self.LngSelect.v_model = name
             elif any(ext in lname for ext in ['lat', 'latitude', 'y_coord', 'ycoord']):
                 self.LatSelect.v_model = name
+        
+        return self
                 
     def _clear_select(self):
-        """clear the select v_model"""
+        """clear the selects components"""
+        
         self.IdSelect.items = [] # all the others are listening to this one 
         self.IdSelect.v_model = self.LngSelect.v_model = self.LatSelect.v_model = None
         
-        return 
+        return self
     
     def _on_select_change(self, change):
+        """change the v_model value when a select is changed"""
         
         name = change['owner']._metadata['name']
         self._set_value(name, change['new'])
         
-        return
+        return self
         
     def _set_value(self, name, value):
-        
         """ set the value in the json dictionary"""
+        
         tmp = json.loads(self.v_model)
         tmp[name] = value
         self.v_model = json.dumps(tmp)
         
-        return
+        return self
     
     def get_v_model(self):
-        """get the v_model as a dict"""
+        """
+        Return the v_model as a dict
+        
+        Return:
+            (dict): the v_model
+        """
+        
         return json.loads(self.v_model)
     
     def get_pathname(self):
-        """return the pathname from v_model"""
+        """
+        Return the pathname from v_model
+        
+        Return:
+            (str): the v_model pathname
+        """
+        
         return json.loads(self.v_model)['pathname']
     
     def get_id_lbl(self):
-        """return the id column label from v_model"""
+        """
+        Return the id column label from v_model
+        
+        Return:
+            (str): label of the id column
+        """
+        
         return json.loads(self.v_model)['id_column']
     
     def get_lng_lbl(self):
-        """return the longitude column label from v_model"""
+        """
+        Return the longitude column label from v_model
+        
+        Return:
+            (str): label of the longitude column
+        """
+        
         return json.loads(self.v_model)['lng_column']
     
     def get_lat_lbl(self):
-        """return the latitude column label from v_model"""
+        """
+        Return the latitude column label from v_model
+        
+        Return:
+            (str); label of the latitude column
+        """
+        
         return json.loads(self.v_model)['lat_column']
-    
-    
-    class _LocalSelect(v.Select):
-            
-            def __init__(self, metadata, label):
-                
-                super().__init__(
-                    _metadata = {'name': metadata}, 
-                    items     = [], 
-                    label     = label, 
-                    v_model   = None
-                )
 
 class AssetSelect(v.Combobox, SepalWidget):
+    """
+    Custom widget input to select an asset inside the asset folder of the user
     
-    def __init__(self, label = 'Select an asset', folder = None):
+    Args:
+        label (str): the label of the input
+        folder (str): the folder of the user assets
+        default_asset (str): the id of a default asset
+        
+    Attributes:
+        folder (str): the folder of the user assets
+    """
+    
+    def __init__(self, label = 'Select an asset', folder = None, default_asset = None):
         
         # if folder is not set use the root one 
         self.folder = folder if folder else ee.data.getAssetRoots()[0]['id'] + '/'
@@ -344,9 +439,7 @@ class AssetSelect(v.Combobox, SepalWidget):
         # get the list of user asset
         assets = ee.data.listAssets({'parent': self.folder})['assets']
         
-        # would be interesting when it will work
-        #items = [{'text': asset['name'].replace(self.folder, ''), 'value': asset['name']} for asset in assets]
-        items = [asset['name'] for asset in assets]
+        items = [asset['id'] for asset in assets]
         
         super().__init__(
             clearable       = True,
@@ -356,6 +449,72 @@ class AssetSelect(v.Combobox, SepalWidget):
             hint            = "select an asset in the list or write a custom asset name. Be careful that you need to have access to this asset to use it",
             persistent_hint = True,
             items           = items,
-            v_model         = None
+            v_model         = default_asset
         )
-        
\ No newline at end of file
+        
+class PasswordField(v.TextField, SepalWidget):
+    """
+    Custom widget to input passwords in text area and 
+    toggle its visibility.
+
+    Args:
+        label (str, optional): Header displayed in text area. Defaults to Password.
+    """
+    def __init__(self, **kwargs):
+        
+        # default behaviour 
+        self.label="Password"
+        self.class_='mr-2'
+        self.v_model=''
+        self.type='password'
+        self.append_icon='mdi-eye-off'
+        
+        # init the widget with the remaining kwargs
+        super().__init__(**kwargs)
+        
+        # bind the js behaviour
+        self.on_event('click:append' ,self._toggle_pwd)
+    
+
+    def _toggle_pwd(self, widget, event, data):
+        """Toggle password visibility when append button is clicked"""
+        
+        if self.type=='text':
+            self.type='password'
+            self.append_icon = 'mdi-eye-off'
+        else:
+            self.type = 'text'
+            self.append_icon = 'mdi-eye'
+            
+class NumberField(v.TextField, SepalWidget):
+    """
+    Custom widget to input numbers in text area and add/substract with single increment.
+
+    Args:
+        max_ (int, optional): Maximum selectable number. Defaults to 10.
+        min_ (int, optional): Minimum selectable number. Defaults to 0.
+
+    """
+    max_ = Int(10).tag(sync=True)
+    min_ = Int(0).tag(sync=True)
+    
+    def __init__(self, **kwargs):
+        
+        self.type='number'
+        self.append_outer_icon='mdi-plus'
+        self.prepend_icon='mdi-minus'
+        self.v_model=0
+        self.readonly=True
+        
+        super().__init__(**kwargs)
+        
+        self.on_event('click:append-outer', self.increment)
+        self.on_event('click:prepend', self.decrement)
+    
+    def increment(self, widget, event, data):
+        """Adds 1 to the current v_model number"""
+        if self.v_model < self.max_: self.v_model+=1
+        
+    def decrement(self, widget, event, data):
+        """Substracts 1 to the current v_model number"""
+        if self.v_model > self.min_: self.v_model-=1
